/**
 * systick.c
 *
 *  Created on: 26.10.2013
 *      Author: ralf
 *
 * \brief
 */
#include <stdint.h>
#include <stdbool.h>

#include "board.h"
#include "chip.h"
#include "global.h"
#include "a2d.h"

volatile uint32_t dwTimeStamp;
volatile bool     systickToggle   = false;
volatile bool     voltageDone     = false;
volatile bool     temperatureDone = false;
volatile bool     checkAdcRdy     = false;


/**
 * void SysTick_Handler (void)
 *
 * \brief Interrupt-Service-Routine fuer den SysTick-Takt. Die Frequenz,
 *        mit der 'SysTick_Handler' aufgerufen wird, wird waehrend der
 *        Initialisierung des Mikrocontrollers in der main()-Funktion
 *        eingestellt.
 *
 *        !!! ACHTUNG !!!
 *        Wir verwenden als Divisor fuer die Initialisierung des SysTick
 *        den Wert 100.000. Hieraus ergibt sich eine Periodendauer von
 *        10s, d.h. der Counter dwTimeStamp wird alle 10s dekrementiert.
 *        Messtechnisch koennen wir in unserem Labor keine kleineren
 *        Werte nachweisen oder bestaetigen.
 *
 * @param
 * @return  void
 */
void SysTick_Handler(void)
{
    dwTimeStamp++;

    if (systickToggle == false)
       systickToggle = true;
    else
       systickToggle = false;

    if ((dwTimeStamp % 100000) == 0)
        checkAdcRdy = true;
}


/**
 * void delayMikros(uint32_t delay)
 *
 * \brief Der Wert 'dwTimeStamp' wird alle 10s dekrementiert, kuerzere
 *        Perioden als 10s lassen sich mit unseren Moeglichkeiten mess-
 *        technisch nicht nachweisen oder bestaetigen. Es lassen sich
 *        daher nur Vielfache von 10s einstellen!
 *
 *        !!! A C H T U N G !!!
 *        Der Prescaler fuer BOARD_MCK wird hierfuer in der main()-Funktion
 *        auf 100.000 eingestellt.
 *
 * @param  uint32_t delay : Vielfaches von 10s
 * @return none
 */
void delayMikros(uint32_t delay)
{
    volatile uint32_t start = dwTimeStamp;
    uint32_t          elapsed;

    // Korrigiert 'delay' so, dass ein 'delay'-Wert von 10 zu einem Systick
    // von 10s fhrt. Auf diese Weise lassen sich aber nur Vielfache von
    // 10s einstellen.
    delay /= 10;
    do
    {
        elapsed = dwTimeStamp;
        elapsed -= start;
    }
    while (elapsed < delay);
}


/**
 * void delayMillis (uint32_t)
 *
 * \brief Der Wert 'dwTimeStamp' wird alle 10s dekrementiert, kuerzere
 *        Perioden als 10s lassen sich mit unseren Moeglichkeiten mess-
 *        technisch nicht nachweisen oder bestaetigen. Durch einen
 *        Korrekturfaktor lassen sich aber Vielfache von 1ms einstellen.
 *
 *        !!! A C H T U N G !!!
 *        Der Prescaler fuer BOARD_MCK wird hierfuer in der main()-Funktion
 *        auf 100.000 eingestellt.
 *
 * @param
 * @return  void
 */
void delayMillis(uint32_t delay)
{
    volatile uint32_t start = dwTimeStamp;
    uint32_t          elapsed;

    // Korrekturfaktor (10s * 100 * delay)
    delay *= 100;
    do
    {
        elapsed = dwTimeStamp;
        elapsed -= start;
    }
    while (elapsed < delay);
}


/**
 * void delaySeconds (uint32_t)
 *
 * \brief Der Wert 'dwTimeStamp' wird alle 10s dekrementiert, kuerzere
 *        Perioden als 10s lassen sich mit unseren Moeglichkeiten mess-
 *        technisch nicht nachweisen oder bestaetigen. Durch einen
 *        Korrekturfaktor lassen sich aber Vielfache von 1ms einstellen.
 *
 *        !!! A C H T U N G !!!
 *        Der Prescaler fuer BOARD_MCK wird hierfuer in der main()-Funktion
 *        auf 100.000 eingestellt.
 *
 * @param
 * @return  void
 */
void delaySeconds(uint32_t delay)
{
    volatile uint32_t start = dwTimeStamp;
    uint32_t          elapsed;

    // Korrekturfaktor (100000 * delay) = 'delay' Sekunden
    delay *= 100000;
    do
    {
        elapsed = dwTimeStamp;
        elapsed -= start;
    }
    while (elapsed < delay);
}

// EOF (systick.c)
